
; Konzept by Mars@AutoIt.de
; Version: 3.Okt.17 - 00:11
; Version: 5.Okt.17 - Abends

Global Const $_ImagePath = @ScriptDir & '\Bims.png' ; Hier ein Testbild einfgen
;~ Global Const $_ImagePath = @ScriptDir & '\Mona.png' ; Hier ein Testbild einfgen
;~ Global Const $_ImagePath = @ScriptDir & '\Mona512.png' ; Hier ein Testbild einfgen
Global Const $_OutputPath = @ScriptDir & '\Output\'     ; Default kann man so lassen
Global Const $_SplitPath = @ScriptDir & '\Split\'       ; Default kann man so lassen
Global Const $_AreaExponent = 0.5 ; Hier kann man was einstellen und die Ergebnisse betrachten. Sollte zwischen 0 und unendlich liegen. Diskussionsbedarf was hier sinnvoll ist.
Global Const $_SplitCount   = 256  ; Anzahl der resultierenden Bilder.

; Was ist das Ziel:
; Die UDF soll es ermglichen ein Bild in kleine Bereiche
; zu teilen deren Inhalt mit einer anderen UDF die noch
; nicht existiert kodiert werden soll. Dafr muss das Bild
; in mglichst "sinnvolle" Teile eingeteilt werden. Die
; I nformation ber die Aufteilung (Position und Gre der
; Einzelteile) soll mglichst kompakt gespeichert werden.
; Es MUSS jede beliebige Bildgre mglich sein. Eine Ein
; teilung in z.B. nur durch 8Px teilbare Gren ist inakzeptabel.

; Was kann die UDF aktuell:
; Eingabe ist ein Bild in einem von GDI+ lesbaren Format.
; Ausgabe ist eine Aufspaltung des Bildes in kleine Einze
; lbilder die jeweils einen hnlichen Informationsgehalt
; besitzen sollen. Mit einer guten Aufteilung wre der er
; ste Schritt getan um das Gesamtbild in kleine Teile zu
; zerlegen die einzeln kodiert werden knnen.

; ######## TO DO LISTE #######
; --- [1] Tree ---
; Die entstehenden Einzelbilder besitzen Ort und Gre. Diese
; Informationen knnten in einer Baumstruktur effizient gespe
; ichert werden. Aktuell ist das nicht der Fall.
;
;                 Bild0
;                   |
;              Vertical Split
;                |         |
;            Bild1        Bild2
;             |               |
;          Horizontal      Vertical
;         |         |       |    |
;       Bild3    Bild4    Bild5 Bild6
;
; Der oben gezeigte Baum ergibt 4 Einzelbilder deren Orte und Gre
; sich vollstndig aus der Gesamtbreite und Gesamthhe des Ursprungsbildes ergibt.
; Ohne Baum mssen diese Infos einzeln gespeichert werden. Der Baum muss
; nicht symmetrisch sein, z.B. kann es sein, dass Bild4 nicht weiter aufgespal
; ten wird, Bild 3, 5 und 6 aber schon.
;
; |----------------|---------------|
; |                |       |       |
; |                | Bild5 | Bild6 |
; |     Bild3      |       |       |
; |                |       |       |
; |----------------|       |       |
; |                |       |       |
; |                |       |       |
; |     Bild4      |       |       |
; |                |       |       |
; |----------------|---------------|
;
; --- [2] Abbruch bzw Errorhandeling bzw Selektion ---
; Aktuell gibt es noch keine gute Methode um zu bestimmen was die Untergrenze fr
; die Bildgre ist. Es werden einfach Bilder nur gespalten sofern sie mindestens
; 2px breit bzw. hoch sind. Das ist aber nicht der richtige Weg glaube ich (Mars).
; Es muss eine Methode her die sinnvoll entscheidet ob ein Bild gespalten werden
; sollte und ggf. ob ein anderes Bild gespalten werden sollte. Aktuell wird einfa
; ch das Bild maximaler Komplexitt geteilt, ggf. ein Fallback, sofern dieses Bil
; d nicht geteilt werden kann ein anderes auswhlen.

; Methode:
; ## 1. Komplexitt bzw. Informationsgehalt eines Bildes/Px ###
;    Es wird eine RGB Durchschnittsfarbe aller Pixel berechnet.
;    Anschlieend wird die Standardabweichung (wieder RGB) der
;    Einzelfarben um diesen Mittelwert berechnet. Diese Standa
;    rdabweichung wird fr R, G, B einzeln berechnet und zu ei
;    ner "Komplexitt pro Pixel" welche zwischen 0 und 1 liegt
;    zusammengefasst. Siehe _GDIPlus_BitmapCalcComplexityRGB
; ## 2. Komplexitt bzw. Informationsgehalt eines Bildes ######
;    Nach Punkt 1 wird die Komplexitt/Px bestimmt. Jetzt hat
;    das betrachtete Bild aber auch eine Breite und eine Hhe
;    und damit eine Pixelanzahl. Um eine Art Informationsgeha
;    lt fr das Gesamtbild zu bekommen wird die Komplexitt p
;    ro Pixel mit der Anzahl Pixel hoch $_AreaExponent genutzt.
;    Prinzipiell reicht das einfache Multiplizieren mit der P
;    ixelzahl, allerdings hat sich gezeigt, dass verschiedene
;    Exponenten (z.B. 0.5) bessere Ergebnisse bei der Findung
;    komplexer aufgebauter Bildgebiete liefert. Eine gnzlich
;    andere Formel fr Punkt 1 knnte hier alles umwerfen.
; ## 3. Aufspaltung eines Bildes ##############################
;    Das Bild wird einmal vertikal und einmal horizontal gespa
;    lten und jeweils die damit einhergehenden Einzelkomplexit
;    ten berechnet. Da das Ziel ist das Bild in "einfache" und
;    "komplexe" Gebiete zu teilen wird der Versuch verwendet d
;    er die grere Aufspaltung liefert. Anschlieend wird das
;    Ursprungsbild mit den 2 neuen Bildern ersetzt und der Vorg
;    ang rekursiv fortgesetzt. Aus der Liste der Einzelbilder
;    wird immer das Einzelbild maximaler Komplexitt geteilt.

; Ideen:
; ## 1. Zur Bestimmung der Komplexitt von der verwendeten Met
;       ode Abstand nehmen und in ein anderes Farbsystem wechseln. (z.B. Lab)
; ## 2. Komplett andere Teilungsmethode finden die auch auf RGB basiert.

; Includes - Obvious
#include <Array.au3> ; Bleibt drin, falls irgendwer _ArrayDisplay braucht.
#include <GDIPlus.au3> ; Logisch, es geht um Bilder...

; Hilfsarray fr Einzelbilder $aaIMG enthlt einige Grundinformationen (kann ggf angepasst werden, sofern sinnvoll)
Global Const $_SubIMG_BMP = 0
Global Const $_SubIMG_X = 1
Global Const $_SubIMG_Y = 2
Global Const $_SubIMG_W = 3
Global Const $_SubIMG_H = 4
Global Const $_SubIMG_Complexity = 5

; ######################### Testgebiet #############################

_GDIPlus_Startup()
Local $hGDIPlusImg = _GDIPlus_ImageLoadFromFile($_ImagePath)
Local $aSize       = _GDIPlus_ImageGetDimension($hGDIPlusImg)

Local $aSplitBMP               = _GDIPlus_BitmapGenerateSplitTree($hGDIPlusImg, $_SplitCount) ; Array aus $aaIMG = [$aIMG1, $aIMG2, ...]
Local $hBMP_Assembled          = _GDIPlus_SplitBitmapArrayAssemble($aSplitBMP, $aSize[0], $aSize[1], 0)
Local $hBMP_AssembledWithLines = _GDIPlus_SplitBitmapArrayAssemble($aSplitBMP, $aSize[0], $aSize[1], 1)
Local $hBMP_AssembledWithColor = _GDIPlus_SplitBitmapArrayAssemble($aSplitBMP, $aSize[0], $aSize[1], 2)

If Not FileExists($_OutputPath) Then DirCreate($_OutputPath)
If Not FileExists($_SplitPath) Then DirCreate($_SplitPath)

; Alle Einzelbilder einzeln speichern damit man sieht was bei dem Teilen herauskommt
For $i = 0 To UBound($aSplitBMP) - 1 Step 1
	_GDIPlus_ImageSaveToFile(($aSplitBMP[$i])[$_SubIMG_BMP], $_SplitPath & $i & '.png')
Next

_GDIPlus_ImageSaveToFile($hBMP_Assembled,          $_OutputPath & 'Assembled' & '.png') ; Zusammengesetztes Bild zur berprfung ob alles passt.
_GDIPlus_ImageSaveToFile($hBMP_AssembledWithLines, $_OutputPath & 'AssembledWL' & '.png') ; Nochmal "With Lines" damit man die Bereiche sieht.
_GDIPlus_ImageSaveToFile($hBMP_AssembledWithColor, $_OutputPath & 'AssembledWC' & '.png') ; Nochmal mit einfarbigen Flchen.

_GDIPlus_ImageDispose($hGDIPlusImg)
_GDIPlus_BitmapDispose($hBMP_Assembled)
_GDIPlus_BitmapDispose($hBMP_AssembledWithLines)
_GDIPlus_BitmapDispose($hBMP_AssembledWithColor)
_GDIPlus_SplitBitmapArrayDispose($aSplitBMP)
_GDIPlus_Shutdown()

; ###################################################################

Func SubImage($hBMP, $x = -1, $y = -1, $w = -1, $h = -1, $iCol = -1)
	Local $aIMG = [$hBMP, $x, $y, $w, $h, $iCol] ; Hier sieht man die Struktur die im Folgenden berall verwendet wird.
	If $x = -1  Or $y = -1 Then Return SetError(1, 0, 'Ungltive X- oder Y-Koordinate')
	If $w = -1 Or $h = -1 Or $iCol = -1 Then ; Breite, Hhe oder Durchschnittsfarbe lassen sich an hBMP ermitteln.
		Local $aTMP = _GDIPlus_ImageGetDimension($hBMP)
		$aIMG[$_SubIMG_W] = $aTMP[0]
		$aIMG[$_SubIMG_H] = $aTMP[1]
		$aIMG[$_SubIMG_Complexity] = _GDIPlus_BitmapCalcComplexityRGB($hBMP) * ($aTMP[0] * $aTMP[1]) ^ $_AreaExponent
	EndIf
	Return $aIMG
EndFunc

Func _GDIPlus_SplitBitmapArrayDispose(ByRef $aSplitBMP)
	For $i = 0 To UBound($aSplitBMP) - 1 Step 1
		_GDIPlus_BitmapDispose(($aSplitBMP[$i])[$_SubIMG_BMP])
	Next
EndFunc

Func _GDIPlus_SplitBitmapArrayAssemble(ByRef $aSplitBMP, $iW, $iH, $iMode = 0)
	; iMode:
	; 0 - Normal zusammensetzen, ohne alles
	; 1 - Mit Linien um Bereiche zu erkennen
	; 2 - Einfarbige Flchen
	Local $hBMP_Assembled = _GDIPlus_BitmapCreateFromScan0($iW, $iH)
	Local $hGFX = _GDIPlus_ImageGetGraphicsContext($hBMP_Assembled)
	Local $hPen = _GDIPlus_PenCreate(0x80000000)
	Local $hBRU = _GDIPlus_BrushCreateSolid()
	_GDIPlus_PenSetAlignment($hPen, 1)
	For $i = 0 To UBound($aSplitBMP) - 1 Step 1
;~ 		ConsoleWrite($i & ' X:' & ($aSplitBMP[$i])[$_SubIMG_X] & ' Y: ' & ($aSplitBMP[$i])[$_SubIMG_Y] & ' W: ' & ($aSplitBMP[$i])[$_SubIMG_W] & ' H: ' & ($aSplitBMP[$i])[$_SubIMG_H] & @CRLF)
		Switch $iMode
			Case 0
				_GDIPlus_GraphicsDrawImageRect($hGFX, ($aSplitBMP[$i])[$_SubIMG_BMP], ($aSplitBMP[$i])[$_SubIMG_X], ($aSplitBMP[$i])[$_SubIMG_Y], ($aSplitBMP[$i])[$_SubIMG_W], ($aSplitBMP[$i])[$_SubIMG_H])
			Case 1
				_GDIPlus_GraphicsDrawImageRect($hGFX, ($aSplitBMP[$i])[$_SubIMG_BMP], ($aSplitBMP[$i])[$_SubIMG_X], ($aSplitBMP[$i])[$_SubIMG_Y], ($aSplitBMP[$i])[$_SubIMG_W], ($aSplitBMP[$i])[$_SubIMG_H])
				_GDIPlus_GraphicsDrawRect($hGFX, ($aSplitBMP[$i])[$_SubIMG_X], ($aSplitBMP[$i])[$_SubIMG_Y], ($aSplitBMP[$i])[$_SubIMG_W] - 1, ($aSplitBMP[$i])[$_SubIMG_H] - 1, $hPen)
			Case 2
				_GDIPlus_BrushSetSolidColor($hBRU, _GDIPlus_BitmapCalcMainColor(($aSplitBMP[$i])[$_SubIMG_BMP]))
				_GDIPlus_GraphicsFillRect($hGFX, ($aSplitBMP[$i])[$_SubIMG_X], ($aSplitBMP[$i])[$_SubIMG_Y], ($aSplitBMP[$i])[$_SubIMG_W], ($aSplitBMP[$i])[$_SubIMG_H], $hBRU)
		EndSwitch
	Next
	_GDIPlus_GraphicsDispose($hGFX)
	_GDIPlus_BrushDispose($hBRU)
	_GDIPlus_PenDispose($hPen)
	Return $hBMP_Assembled
EndFunc

Func _GDIPlus_BitmapGenerateSplitTree($hBMP, $nSingleBitmaps = 5)
	Local $t = TimerInit(), $xReturn = ''
	Local $aSize = _GDIPlus_ImageGetDimension($hBMP), $aTMP
	Local $aaIMG = [SubImage(_GDIPlus_BitmapCloneArea($hBMP, 0, 0, $aSize[0], $aSize[1]), 0, 0)] ; Grundbild als einziges Bild in aaIMG
	For $i = 1 To $nSingleBitmaps - 1 Step 1
		$xReturn = OneSplit($aaIMG) ; Das Bild maximaler Komplexitt wird jeweils durch 2 Bilder ersetzt.
		If @error Then
			ConsoleWrite('! ' & $xReturn & @CRLF)
			ExitLoop ; Notfall ist eingetreten, sollte nicht passieren.
		EndIf
	Next
	ConsoleWrite('### Imagesplit ###' & @CRLF & '# Count: ' & UBound($aaIMG) & @CRLF & '# Time:  ' & Round(TimerDiff($t)/1000, 2) & 's' & @CRLF & '##################' & @CRLF)
	Return $aaIMG
EndFunc

Func OneSplit(ByRef $aBMP)
	Local $iUBound = UBound($aBMP)
	Local $aComplexity[$iUBound], $aTMP, $aIMG
	Local $iIndexMaxComplexity = 0, $iMaxComplexity = -1

	; Komplexitt aller Teilbilder betrachten und Teilbild maximaler Komplexitt finden
	For $i = 0 To $iUBound - 1 Step 1
		If ($aBMP[$i])[$_SubIMG_Complexity] > $iMaxComplexity And ($aBMP[$i])[$_SubIMG_W] > 1 And ($aBMP[$i])[$_SubIMG_H] > 1 Then
			$iMaxComplexity = ($aBMP[$i])[$_SubIMG_Complexity]
			$iIndexMaxComplexity = $i
		EndIf
	Next

	; Das Teilbild einmal Horizontal und einmal Vertikal splitten.
	$aIMG = $aBMP[$iIndexMaxComplexity]
	Local $aBMP_H = _GDIPlus_BitmapSplitHorizontal($aIMG[$_SubIMG_BMP])
	Local $aBMP_V = _GDIPlus_BitmapSplitVertical($aIMG[$_SubIMG_BMP])

	;Altes Teilbild in die Tonne. Wird durch 2 neue ersetzt.
	_GDIPlus_BitmapDispose($aIMG[$_SubIMG_BMP])

	; Schauen wo die Komplexitt pro Pixel maximal ansteigt (sinnvoll? oder lieber andere Methode?)
	Local $iComplexityH = _GDIPlus_BitmapCalcComplexityDifferenceRGB($aBMP_H[0], $aBMP_H[1])
	Local $iComplexityV = _GDIPlus_BitmapCalcComplexityDifferenceRGB($aBMP_V[0], $aBMP_V[1])

	; Folgendes lsst sich einfach zusammenfassen, falls man Einzelheiten ndern will bleibt es aber vorerst getrennt.
	If $iComplexityH > $iComplexityV Then ; Horizontal splitten
		$aTMP = _GDIPlus_ImageGetDimension($aBMP_H[0])
		If $aTMP[0] <= $aTMP[1] * 8 Then ; Grenzwert ist, dass ein Bild nicht mehr als 8x so breit wie hoch sein darf und umgekehrt.
			_GDIPlus_BitmapDispose($aBMP_V[0])
			_GDIPlus_BitmapDispose($aBMP_V[1])
			$aBMP[$iIndexMaxComplexity] = SubImage($aBMP_H[0], $aIMG[$_SubIMG_X], $aIMG[$_SubIMG_Y], -1)
			_Insert1D($aBMP, SubImage($aBMP_H[1], $aIMG[$_SubIMG_X], $aIMG[$_SubIMG_Y] + $aTMP[1], -1), $iIndexMaxComplexity + 1)
		Else
			$aTMP = _GDIPlus_ImageGetDimension($aBMP_V[0])
			_GDIPlus_BitmapDispose($aBMP_H[0])
			_GDIPlus_BitmapDispose($aBMP_H[1])
			$aBMP[$iIndexMaxComplexity] = SubImage($aBMP_V[0], $aIMG[$_SubIMG_X], $aIMG[$_SubIMG_Y], -1)
			_Insert1D($aBMP, SubImage($aBMP_V[1], $aIMG[$_SubIMG_X] + $aTMP[0], $aIMG[$_SubIMG_Y], -1), $iIndexMaxComplexity + 1)
		EndIf
	Else ; Vertikal splitten
		$aTMP = _GDIPlus_ImageGetDimension($aBMP_V[0])
		If $aTMP[1] <= $aTMP[0] * 8 Then
			_GDIPlus_BitmapDispose($aBMP_H[0])
			_GDIPlus_BitmapDispose($aBMP_H[1])
			$aBMP[$iIndexMaxComplexity] = SubImage($aBMP_V[0], $aIMG[$_SubIMG_X], $aIMG[$_SubIMG_Y], -1)
			_Insert1D($aBMP, SubImage($aBMP_V[1], $aIMG[$_SubIMG_X] + $aTMP[0], $aIMG[$_SubIMG_Y], -1), $iIndexMaxComplexity + 1)
		Else
			$aTMP = _GDIPlus_ImageGetDimension($aBMP_H[0])
			_GDIPlus_BitmapDispose($aBMP_V[0])
			_GDIPlus_BitmapDispose($aBMP_V[1])
			$aBMP[$iIndexMaxComplexity] = SubImage($aBMP_H[0], $aIMG[$_SubIMG_X], $aIMG[$_SubIMG_Y], -1)
			_Insert1D($aBMP, SubImage($aBMP_H[1], $aIMG[$_SubIMG_X], $aIMG[$_SubIMG_Y] + $aTMP[1], -1), $iIndexMaxComplexity + 1)
		EndIf
	EndIf
	Return 'Alles Okay'
EndFunc

Func _Insert1D(ByRef $aArray, $iItem, $iPos) ; Rocket Science
	ReDim $aArray[UBound($aArray) + 1]
	For $i = UBound($aArray) - 1 To $iPos  Step -1
		$aArray[$i] = $aArray[$i - 1]
	Next
	$aArray[$iPos] = $iItem
EndFunc

; Return: Kommazahl die den Komplexittsunterschied zwischen zwei Bildern angibt.
Func _GDIPlus_BitmapCalcComplexityDifferenceRGB($hBMP1, $hBMP2)
	Local $1 = _GDIPlus_BitmapCalcComplexityRGB($hBMP1)
	Local $2 = _GDIPlus_BitmapCalcComplexityRGB($hBMP2)
	Return Abs($1 - $2)
EndFunc

; Return: String "0xAARRGGBB", wobei hier der Alphakanal = 255 gesetzt wurde um Rechenzeit zu sparen.
Func _GDIPlus_BitmapCalcMainColor($hBMP)
	Local $vBMP = _GDIPlus_BitmapToStruct($hBMP)
	Local $R, $G, $B, $iPx = DllStructGetSize($vBMP)/4, $iStABWR, $iStABWG, $iStABWB
	For $i = 1 To $iPx * 4 Step 4
		$B += DllStructGetData($vBMP, 1, $i)
		$G += DllStructGetData($vBMP, 1, $i+1)
		$R += DllStructGetData($vBMP, 1, $i+2)
	Next
	$R /= $iPx
	$G /= $iPx
	$B /= $iPx
	Return '0xFF' & Hex(Int(Round($R, 0)), 2) & Hex(Int(Round($G, 0)), 2) & Hex(Int(Round($B, 0)), 2)
EndFunc

; Return: Kommazahl zwischen 0 und 1 die die Komplexitt pro Pixel angibt.
; 0 -> Einfarbiges Bild
; 1 -> Maximalwert (nur erreichbar bei Schwarzwei mit exakt gleicher Anzahl schwarzer wie weier Pixel)
Func _GDIPlus_BitmapCalcComplexityRGB($hBMP)
	Local $vBMP = _GDIPlus_BitmapToStruct($hBMP)
	Local $R, $G, $B, $iPx = DllStructGetSize($vBMP)/4, $iStABWR, $iStABWG, $iStABWB
	For $i = 1 To $iPx * 4 Step 4
		$B += DllStructGetData($vBMP, 1, $i)
		$G += DllStructGetData($vBMP, 1, $i+1)
		$R += DllStructGetData($vBMP, 1, $i+2)
	Next
	$R /= $iPx
	$G /= $iPx
	$B /= $iPx
	For $i = 1 To $iPx * 4 Step 4
		$iStABWB += (DllStructGetData($vBMP, 1, $i) - $B)^2
		$iStABWG += (DllStructGetData($vBMP, 1, $i+1) - $G)^2
		$iStABWR += (DllStructGetData($vBMP, 1, $i+2) - $R)^2
	Next
;~ 	$iStABWR = ($iStABWR / $iPx)^0.5 / 127.5 / 3^0.5 ; StABW liegt mit dieser Rechnung zwischen 0 und 127.5
;~ 	$iStABWG = ($iStABWG / $iPx)^0.5 / 127.5 / 3^0.5 ; Im Endergebnis soll sie zwischen 0 und 1 liegen. Daher wird auf den
;~ 	$iStABWB = ($iStABWB / $iPx)^0.5 / 127.5 / 3^0.5 ; Bereich 0 bis SQRT(1/3) skaliert.
;~ 	Return ($iStABWR^2 + $iStABWG^2 + $iStABWB^2)^0.5
	Return (($iStABWR + $iStABWG + $iStABWB) / $iPx / 48768.75 )^0.5 ; Zusammenfassung der 4 oberen Zeilen.
EndFunc

; Return: [0] - Linksoben
;         [1] - Rechtsoben
;         [2] - Linksunten
;         [3] - Rechtsunten
Func _GDIPlus_BitmapSplitQuad($hBMP) ; Unbenutzt, zumindest bisher.
	Local $aBMP_V = _GDIPlus_BitmapSplitVertical($hBMP)
	Local $aBMP_L = _GDIPlus_BitmapSplitHorizontal($aBMP_V[0])
	Local $aBMP_R = _GDIPlus_BitmapSplitHorizontal($aBMP_V[1])
	_GDIPlus_BitmapDispose($aBMP_V[0])
	_GDIPlus_BitmapDispose($aBMP_V[1])
	Local $aRet = [$aBMP_L[0], $aBMP_R[0], $aBMP_L[1], $aBMP_R[1]]
	Return $aRet
EndFunc

; Return: [0] - Linkes Halbbild
;         [1] - Rechtes Halbbild
Func _GDIPlus_BitmapSplitVertical($hBMP)
	Local $aSize = _GDIPlus_ImageGetDimension($hBMP)
	Local $hBMP_L = _GDIPlus_BitmapCloneArea($hBMP, 0, 0, Round($aSize[0]/2, 0), $aSize[1])
	Local $hBMP_R = _GDIPlus_BitmapCloneArea($hBMP, Round($aSize[0]/2, 0), 0, $aSize[0] - Round($aSize[0]/2, 0), $aSize[1])
	Local $aRet = [$hBMP_L, $hBMP_R]
	Return $aRet
EndFunc

; Return: [0] - Oberes Halbbild
;         [1] - Unteres Halbbild
Func _GDIPlus_BitmapSplitHorizontal($hBMP)
	Local $aSize = _GDIPlus_ImageGetDimension($hBMP)
	Local $hBMP_O = _GDIPlus_BitmapCloneArea($hBMP, 0, 0, $aSize[0], Round($aSize[1]/2, 0))
	Local $hBMP_U = _GDIPlus_BitmapCloneArea($hBMP, 0, Round($aSize[1]/2, 0), $aSize[0], $aSize[1] - Round($aSize[1]/2, 0))
	Local $aRet = [$hBMP_O, $hBMP_U]
	Return $aRet
EndFunc

; Return: DllStruct byte[Size*8] sodass man die RGB Werte einfach via Struct erreichen kann.
Func _GDIPlus_BitmapToStruct($hBMP) ; BGRA8888
	Local $aSize = _GDIPlus_ImageGetDimension($hBMP)
	If @error Then ConsoleWrite(@error & ' ' & @extended & @CRLF)
	Local $vBMP8888 = DllStructCreate('byte[' & 4 * $aSize[0] * $aSize[1] & ']')
	Local $vBMP = DllStructCreate('int[' & $aSize[0] * $aSize[1] & ']', DllStructGetPtr($vBMP8888))
	Local $hScan0 = _GDIPlus_BitmapCreateFromScan0($aSize[0], $aSize[1], $GDIP_PXF32ARGB, $aSize[0] * 4, DllStructGetPtr($vBMP))
	Local $hGFX = _GDIPlus_ImageGetGraphicsContext($hScan0)
	_GDIPlus_GraphicsDrawImageRect($hGFX, $hBMP, 0, 0, $aSize[0], $aSize[1])
	_GDIPlus_GraphicsDispose($hGFX)
	_GDIPlus_BitmapDispose($hScan0)
	Return $vBMP8888
EndFunc